package com.muyu.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.model.CategoryInfoSaveModel;
import com.muyu.product.domain.resp.CategoryResp;
import com.muyu.product.mapper.CategoryInfoMapper;
import com.muyu.product.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 品类信息Service业务层处理
 *
 * @author Saisai
 * @date 2024-02-29
 */
@Slf4j
@Service
public class CategoryInfoServiceImpl extends ServiceImpl<CategoryInfoMapper, CategoryInfo> implements ICategoryInfoService {
    @Autowired
    private CategoryInfoMapper categoryInfoMapper;
    @Autowired
    private ICategoryAttributeService categoryAttributeService;
    @Autowired
    private ICategoryAttributeGroupService categoryAttributeGroupService;
    @Autowired
    private IBrandCategoryService brandCategoryService;
    @Autowired
    private IAttributeService attributeService;
    @Autowired
    private IAttributeGroupService groupService;
    @Autowired
    private IBrandService brandService;


    /**
     * 查询品类信息
     *
     * @param id 品类信息主键
     * @return 品类信息
     */
    @Override
    public CategoryInfo selectCategoryInfoById(Long id) {
        return categoryInfoMapper.selectCategoryInfoById(id);
    }

    /**
     * 查询品类信息列表
     *
     * @param categoryInfoSaveModel 品类信息
     * @return 品类信息
     */
    @Override
    public List<CategoryResp> selectCategoryInfoList(CategoryInfoSaveModel categoryInfoSaveModel) {
        //三大响应结果集
//        List<CategoryAttribute> categoryAttributes = CategoryAttribute.saveBuilder(categoryInfoSaveModel, SecurityUtils::getUsername);
//        List<CategoryAttributeGroup> categoryAttributeGroups = CategoryAttributeGroup.saveBuilderList(categoryInfoSaveModel, SecurityUtils::getUsername);
//        List<BrandCategory> brandCategories = BrandCategory.saveBuilderList(categoryInfoSaveModel, SecurityUtils::getUsername);
//        List<CategoryInfo> categoryInfos = null;


        List<CategoryResp> list = categoryInfoMapper.selectCategoryInfoList(CategoryInfo.saveBuilder(categoryInfoSaveModel)).stream().map(categoryInfo -> {
            CategoryInfoSaveModel infoSaveModel = new CategoryInfoSaveModel();
            infoSaveModel.setId(categoryInfo.getId());
            List<Long> attrIds = categoryAttributeService.list(new LambdaQueryWrapper<CategoryAttribute>().eq(CategoryAttribute::getCategoryId, categoryInfo.getId())).stream().map(CategoryAttribute::getAttributeId).toList();
            List<Long> groupIds = categoryAttributeGroupService.list(new LambdaQueryWrapper<CategoryAttributeGroup>().eq(CategoryAttributeGroup::getCategoryId, categoryInfo.getId())).stream().map(CategoryAttributeGroup::getAttributeGroupId).toList();
            List<Long> brandIds = brandCategoryService.list(new LambdaQueryWrapper<BrandCategory>().eq(BrandCategory::getCategoryId, categoryInfo.getId())).stream().map(BrandCategory::getBrandId).toList();
            CategoryResp build = CategoryResp.builder()
                    .id(categoryInfo.getId())
                    .parentId(categoryInfo.getParentId())
                    .name(categoryInfo.getName())
                    .image(categoryInfo.getImage())
                    .status(categoryInfo.getStatus())
                    .brandList(brandIds.isEmpty() ? null : brandService.listByIds(brandIds))
                    .checkAttributeList(attrIds.isEmpty() ? null : attributeService.listByIds(attrIds))
                    .checkedAttributeGroupList(groupIds.isEmpty() ? null : groupService.listByIds(groupIds))
                    .build();
            return build;
        }).toList();

        return list;
    }

    /**
     * 新增品类信息
     *
     * @param categoryInfoSaveModel 品类信息
     * @return 结果
     */
    @Override
    public int insertCategoryInfo(CategoryInfoSaveModel categoryInfoSaveModel) {
        CategoryInfo categoryInfo = CategoryInfo.saveBuilder(categoryInfoSaveModel);
        int i = categoryInfoMapper.insertCategoryInfo(categoryInfo);
        categoryInfoSaveModel.setId(categoryInfo.getId());
        categoryAttributeService.saveBatch(CategoryAttribute.saveBuilder(categoryInfoSaveModel, SecurityUtils::getUsername));
        categoryAttributeGroupService.saveBatch(CategoryAttributeGroup.saveBuilderList(categoryInfoSaveModel, SecurityUtils::getUsername));
        brandCategoryService.saveBatch(BrandCategory.saveBuilderList(categoryInfoSaveModel, SecurityUtils::getUsername));
        return i;
    }

    /**
     * 修改品类信息
     *
     * @param categoryInfoSaveModel 品类信息
     * @return 结果
     */
    @Override
    public boolean updateCategoryInfo(CategoryInfoSaveModel categoryInfoSaveModel) {
        CategoryInfo categoryInfo = CategoryInfo.saveBuilder(categoryInfoSaveModel);

        boolean updated = this.update(new LambdaQueryWrapper<CategoryInfo>().eq(CategoryInfo::getId, categoryInfo.getId()));
//        LambdaQueryWrapper<CategoryAttribute> attributeWrapper = new LambdaQueryWrapper<CategoryAttribute>().eq(CategoryAttribute::getCategoryId,categoryInfoSaveModel.getId());
        //删除中间表数据
        categoryAttributeService.remove(new LambdaQueryWrapper<CategoryAttribute>()
                .eq(CategoryAttribute::getCategoryId, categoryInfoSaveModel.getId())
        );
        //添加新数据
        categoryAttributeService.saveBatch(CategoryAttribute.saveBuilder(categoryInfoSaveModel, SecurityUtils::getUsername));
        //修改属性组中间表
//        LambdaQueryWrapper<CategoryAttributeGroup> groupWrapper = new LambdaQueryWrapper<CategoryAttributeGroup>();
        categoryAttributeGroupService.remove(new LambdaQueryWrapper<CategoryAttributeGroup>()
                .eq(CategoryAttributeGroup::getCategoryId, categoryInfoSaveModel.getId())
        );
        categoryAttributeGroupService.saveBatch(
                CategoryAttributeGroup.saveBuilderList(categoryInfoSaveModel, SecurityUtils::getUsername)
        );
//        LambdaQueryWrapper<BrandCategory> brandWrapper = new LambdaQueryWrapper<BrandCategory>();
        brandCategoryService.remove(
                new LambdaQueryWrapper<BrandCategory>()
                        .eq(BrandCategory::getCategoryId, categoryInfoSaveModel.getId())
        );
        brandCategoryService.saveBatch(
                BrandCategory.saveBuilderList(categoryInfoSaveModel, SecurityUtils::getUsername)
        );

        return updated;
    }

    /**
     * 批量删除品类信息
     *
     * @param ids 需要删除的品类信息主键
     * @return 结果
     */
    @Override
    public int deleteCategoryInfoByIds(Long[] ids) {
        return categoryInfoMapper.deleteCategoryInfoByIds(ids);
    }

    /**
     * 删除品类信息信息
     *
     * @param id 品类信息主键
     * @return 结果
     */
    @Override
    public int deleteCategoryInfoById(Long id) {
        categoryAttributeService.remove(new LambdaQueryWrapper<CategoryAttribute>().eq(CategoryAttribute::getCategoryId, id));
        categoryAttributeGroupService.remove(new LambdaQueryWrapper<CategoryAttributeGroup>().eq(CategoryAttributeGroup::getCategoryId, id));
        brandCategoryService.remove(new LambdaQueryWrapper<BrandCategory>().eq(BrandCategory::getCategoryId, id));
        return categoryInfoMapper.deleteCategoryInfoById(id);
    }

    /**
     * 品类数据添加
     *
     * @param categoryInfoSaveModel
     * @return
     */
    @Override
    public boolean save(CategoryInfoSaveModel categoryInfoSaveModel) {
        //通过品类信息构建体构建品类对象
        CategoryInfo categoryInfo = CategoryInfo.saveCategoryBuilder(categoryInfoSaveModel);
        //调用重载方法
        boolean save = this.save(categoryInfo);
        //获取当前品类主键
        Long id = categoryInfo.getId();
        //获取已选属性主键集
        List<Long> checkedAttributeIds = categoryInfoSaveModel.getCheckedAttributeIds();
        //判断已选属性集是否为空
        if (checkedAttributeIds != null && !checkedAttributeIds.isEmpty()) {
            categoryAttributeService.saveBatch(
                    checkedAttributeIds.stream()
                            .map(attributeId -> CategoryAttribute.categoryBuilder(id, attributeId, SecurityUtils::getUsername)).toList()
            );
        }
        List<Long> attributeGroupIds = categoryInfoSaveModel.getCheckedAttributeGroupIds();
        if (attributeGroupIds != null && !attributeGroupIds.isEmpty()) {
            categoryAttributeGroupService.saveBatch(
                    attributeGroupIds.stream().
                            map(groupId -> CategoryAttributeGroup.saveBuilder(id, groupId, SecurityUtils::getUsername)).toList()
            );
        }
        return save;
    }
}
